//==============================================================================
unit Test8087CWRoundInfo;
//==============================================================================

//==============================================================================
interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls,
  FloatCtrl;
//==============================================================================

//------------------------------------------------------------------------------
const
  S_ROUND_UP = 'Round up (justified)';

//------------------------------------------------------------------------------
var
  DIV_2: Extended = 0.5;

//------------------------------------------------------------------------------
const
  S_FLOAT_TPL = '#,##0.000000000000000;';
  S_FLOAT_FORMAT_TPL =
    '''+''' + S_FLOAT_TPL + '''-''' + S_FLOAT_TPL + '''''' + S_FLOAT_TPL;

//------------------------------------------------------------------------------
type
  TFormDlgRoundInfo = class(TForm)
    MemoTop: TMemo;
    ButtonOk: TButton;
    ButtonRoundUp: TButton;
    ButtonUse8087CW: TButton;
    procedure ButtonRoundUpClick(Sender: TObject);
    procedure ButtonUse8087CWClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    fFloatControl: TFloatControlVerbose;
  public
    procedure ShowRoundControl(
      const aFloatControl: TFloatControlVerbose;
      const aRoundUp: boolean = FALSE);
  end;

//------------------------------------------------------------------------------
var
  FormDlgRoundInfo: TFormDlgRoundInfo;

//==============================================================================
implementation
{$R *.dfm}
uses
  Math;
//==============================================================================

//------------------------------------------------------------------------------
procedure TFormDlgRoundInfo.ButtonRoundUpClick(Sender: TObject);
begin
  ShowRoundControl(fFloatControl, TRUE);
end;

//------------------------------------------------------------------------------
procedure TFormDlgRoundInfo.ButtonUse8087CWClick(Sender: TObject);
begin
  ShowRoundControl(fFloatControl);
end;

//------------------------------------------------------------------------------
procedure TFormDlgRoundInfo.FormCreate(Sender: TObject);
begin
  MemoTop.Font.Color := clBlack;
end;

//------------------------------------------------------------------------------
procedure TFormDlgRoundInfo.ShowRoundControl(
  const aFloatControl: TFloatControlVerbose;
  const aRoundUp: boolean = FALSE);
const
  S_FLOATSEPARATOR = ' => ';
var
  a_DecimalIndex: integer;

  procedure Round_Loop(const aSmallerZero: boolean = TRUE);
  var
    a_Number: integer;
    a_RoundedNumber: integer;
    a_String: string;
    a_Factor: Extended;
    a_Value: Extended;
    a_RoundedValue: Extended;
    a_Char: Char;
    a_Pos: integer;
  begin
    if ( aSmallerZero ) then
      a_Factor := Power(10, -a_DecimalIndex)
    else
      a_Factor := Power(10,  a_DecimalIndex);
    a_String := 'Decimals: ' + FloatToStr(a_Factor);
    MemoTop.Lines.Add(a_String);
    for a_Number := 8 downto -9 do
    begin
      if aRoundUp then
        a_RoundedNumber := RoundJustified(a_Number + DIV_2)
      else
        a_RoundedNumber := Round(a_Number + DIV_2);

      a_Value         := (a_Number + DIV_2) * a_Factor;
      a_RoundedValue  := a_RoundedNumber    * a_Factor;
      a_String :=
        FormatFloat(S_FLOAT_FORMAT_TPL, a_Value)          + S_FLOATSEPARATOR +
        FormatFloat(S_FLOAT_FORMAT_TPL, a_RoundedValue);
      if ( aSmallerZero ) then
      begin
        a_Pos  := (15 - a_DecimalIndex);
        a_Char := a_String[ Length(a_String) - a_Pos ];
        a_String := a_String +
        '(' + a_Char + ')';
      end
      else
      begin
        a_Pos  := Pos( S_FLOATSEPARATOR, a_String ) + 5;
        a_Char := a_String[ a_Pos ];
        a_String := a_String +
        '(' + a_Char + ')';
      end;
      MemoTop.Lines.Add(a_String);
    end;
  end;

begin
  fFloatControl := aFloatControl;

  MemoTop.Lines.BeginUpdate;
  try

    MemoTop.Lines.Clear;

  // Display actual round mode...
    if aRoundUp then
    begin
      MemoTop.Lines.Add(S_ROUND_UP);
      MemoTop.Font.Color := clGreen;
    end
    else
    begin
      MemoTop.Lines.Add(fFloatControl.RoundControlStr);
      if (not fFloatControl.RoundControlLow) and
         (not fFloatControl.RoundControlHigh) then
        MemoTop.Font.Color := clBlack
      else
        MemoTop.Font.Color := clBlue;
    end;

  // Display rounded values...
    for a_DecimalIndex := 10 downto 1 do
      Round_Loop;
    for a_DecimalIndex := 0 to 6 do
      Round_Loop(FALSE);

    MemoTop.SelStart  := 0;
    MemoTop.SelLength := 0;
  finally
    MemoTop.Lines.EndUpdate;
  end;
end;

end.
